unit BaseProfiler;

//TODO: Add in code to log to a file
//TODO: Add in code to do proper thread-safe nesting (not using TLS)

{$WARN SYMBOL_PLATFORM OFF}

// Notes:
//
// For efficiency, the profiler should build tables of AppDomains, assemblies,
// threads, modules, classes, objects and functions based around their
// IDs (addresses). The various events can be used to indicate when an item
// should be removed or added, or updated.
//
// - a GC will modify various ObjectIDs so use MovedReferences to keep them
//   up to date
// - a function can be pitched and later re-JITted. It will keep the same
//   FunctionID, but this will be invalid between these two events
// - managed threads do not have a 1-1 relationship with OS threads (except
//   in .NET 1.x), so TLS is no good

{$define USE_CRITICAL_SECTION}
{$define NOISE_ON_START_AND_STOP}
{$define UNICODE_CLASS_NAME}
{.$define LOG_COR_ERRORS}

interface

uses
  Windows, ActiveX, Classes, ComObj, CorProf;

// In .NET 1.x managed threads have a 1-1 mapping with OS threads, so
// TLS can be used to store data for each managed thread, like this one.
// In future versions of .NET managed threads will NOT have a 1-1 mapping
// with OS threads and so this code will break because of TLS usage.
// Should use some storage structure that records the managed thread ID
// instead. Thread creation/destruction events give you the managed thread ID
// and ICorProfilerInfo.GetCurrentThreadID always gives you the current thread
threadvar
  NestingLevel: Integer;

type
  TBaseProfiler = class(TComObject, ICorProfilerCallback)
  protected
{$ifdef USE_CRITICAL_SECTION}
    CritSec: TRTLCriticalSection;
{$endif}
    CorProfilerInfo: ICorProfilerInfo;
    EventMask: DWord;

    //Helper methods
    function GetAppDomainFromAppDomainId(AppDomainId: ULONG;
      var AppDomainName: WideString): Boolean;
    function GetAssemblyFromAssemblyId(AssemblyId: ULONG;
      var AssemblyName: WideString): Boolean;
    function GetClassAndMethodFromFunctionId(FunctionId: ULONG;
      var ClassName, MethodName: WideString): Boolean;
    function GetClassFromObjectId(ObjectId: ULONG;
      var ClassName: WideString): Boolean;
    function GetClassFromClassId(ClassId: ULONG;
      var ClassName: WideString): Boolean;
    function GetModuleFromModuleId(ModuleId: ULONG;
      var ModuleName: WideString): Boolean;
    procedure Log(const Msg: String); overload;
    procedure Log(const Msg: String; Args: array of const); overload;
    procedure OnInitialize; virtual;
    procedure OnShutDown; virtual;
    procedure SyncEnter;
    procedure SyncExit;

    //ICorProfilerCallback interface methods

    // STARTUP/SHUTDOWN EVENTS
    procedure Initialize(const pICorProfilerInfoUnk: IUnknown); reintroduce; safecall;
    procedure Shutdown; safecall;
    // COR_PRF_MONITOR_APPDOMAIN_LOADS
    procedure AppDomainCreationStarted(appDomainId: ULONG); safecall;
    procedure AppDomainCreationFinished(appDomainId: ULONG; hrStatus: HResult); safecall;
    procedure AppDomainShutdownStarted(appDomainId: ULONG); safecall;
    procedure AppDomainShutdownFinished(appDomainId: ULONG; hrStatus: HResult); safecall;
    // COR_PRF_MONITOR_ASSEMBLY_LOADS
    procedure AssemblyLoadStarted(assemblyId: ULONG); safecall;
    procedure AssemblyLoadFinished(assemblyId: ULONG; hrStatus: HResult); safecall;
    procedure AssemblyUnloadStarted(assemblyId: ULONG); safecall;
    procedure AssemblyUnloadFinished(assemblyId: ULONG; hrStatus: HResult); safecall;
    // COR_PRF_MONITOR_MODULE_LOADS
    procedure ModuleLoadStarted(moduleId: ULONG); safecall;
    procedure ModuleLoadFinished(moduleId: ULONG; hrStatus: HResult); safecall;
    procedure ModuleUnloadStarted(moduleId: ULONG); safecall;
    procedure ModuleUnloadFinished(moduleId: ULONG; hrStatus: HResult); safecall;
    procedure ModuleAttachedToAssembly(moduleId: ULONG; assemblyId: ULONG); safecall;
    // COR_PRF_MONITOR_CLASS_LOADS
    procedure ClassLoadStarted(classId: ULONG); safecall;
    procedure ClassLoadFinished(classId: ULONG; hrStatus: HResult); safecall;
    procedure ClassUnloadStarted(classId: ULONG); safecall;
    procedure ClassUnloadFinished(classId: ULONG; hrStatus: HResult); safecall;
    // COR_PRF_MONITOR_FUNCTION_UNLOADS
    procedure FunctionUnloadStarted(functionId: ULONG); safecall; //currently inactive
    // COR_PRF_MONITOR_JIT_COMPILATION
    procedure JITCompilationStarted(functionId: ULONG; fIsSafeToBlock: Integer); safecall;
    procedure JITCompilationFinished(functionId: ULONG; hrStatus: HResult; fIsSafeToBlock: Integer); safecall;
    function JITInlining(callerId: ULONG; calleeId: ULONG): {fShouldInline} Bool; safecall;
    // COR_PRF_MONITOR_FUNCTION_UNLOADS
    procedure JITFunctionPitched(functionId: ULONG); safecall;
    // COR_PRF_MONITOR_CACHE_SEARCHES
    function JITCachedFunctionSearchStarted(functionId: ULONG): {bUseCachedFunction} Bool; safecall;
    procedure JITCachedFunctionSearchFinished(functionId: ULONG; AResult: COR_PRF_JIT_CACHE); safecall;
    // COR_PRF_MONITOR_THREADS
    procedure ThreadCreated(threadId: ULONG); safecall;
    procedure ThreadDestroyed(threadId: ULONG); safecall;
    procedure ThreadAssignedToOSThread(managedThreadId: ULONG; osThreadId: ULONG); safecall;
    // COR_PRF_MONITOR_REMOTING
    procedure RemotingClientInvocationStarted; safecall;
    procedure RemotingClientSendingMessage(const pCookie: TGuid; fIsAsync: Bool); safecall;
    procedure RemotingClientReceivingReply(const pCookie: TGuid; fIsAsync: Bool); safecall;
    procedure RemotingClientInvocationFinished; safecall;
    procedure RemotingServerReceivingMessage(const pCookie: TGuid; fIsAsync: Bool); safecall;
    procedure RemotingServerInvocationStarted; safecall;
    procedure RemotingServerInvocationReturned; safecall;
    procedure RemotingServerSendingReply(const pCookie: TGuid; fIsAsync: Bool); safecall;
    // COR_PRF_MONITOR_CODE_TRANSITIONS
    procedure UnmanagedToManagedTransition(functionId: ULONG; reason: COR_PRF_TRANSITION_REASON); safecall;
    procedure ManagedToUnmanagedTransition(functionId: ULONG; reason: COR_PRF_TRANSITION_REASON); safecall;
    // COR_PRF_MONITOR_SUSPENDS
    procedure RuntimeSuspendStarted(suspendReason: COR_PRF_SUSPEND_REASON); safecall;
    procedure RuntimeSuspendFinished; safecall;
    procedure RuntimeSuspendAborted; safecall;
    procedure RuntimeResumeStarted; safecall;
    procedure RuntimeResumeFinished; safecall;
    procedure RuntimeThreadSuspended(threadId: ULONG); safecall;
    procedure RuntimeThreadResumed(threadId: ULONG); safecall;
    // COR_PRF_MONITOR_GC
    procedure MovedReferences(cMovedObjectIDRanges: ULONG;
      var oldObjectIDRangeStart, newObjectIDRangeStart,
          cObjectIDRangeLength: TULONGArray); safecall;
    procedure ObjectsAllocatedByClass(cClassCount: ULONG;
      var classIds, cObjects: TULONGarray); safecall;
    procedure ObjectReferences(objectId: ULONG; classId: ULONG;
      cObjectRefs: ULONG; var objectRefIds: TULONGArray); safecall;
    procedure RootReferences(cRootRefs: ULONG; var rootRefIds: TULONGArray); safecall;
    // COR_PRF_MONITOR_OBJECT_ALLOCATED
    procedure ObjectAllocated(objectId, classId: ULONG); safecall;
    // COR_PRF_MONITOR_EXCEPTIONS
    procedure ExceptionThrown(thrownObjectId: ULONG); safecall;
    procedure ExceptionSearchFunctionEnter(functionId: ULONG); safecall;
    procedure ExceptionSearchFunctionLeave; safecall;
    procedure ExceptionSearchFilterEnter(functionId: ULONG); safecall;
    procedure ExceptionSearchFilterLeave; safecall;
    procedure ExceptionSearchCatcherFound(functionId: ULONG); safecall;
    procedure ExceptionOSHandlerEnter(__unused: ULONG); safecall; //currently inactive
    procedure ExceptionOSHandlerLeave(__unused: ULONG); safecall; //currently inactive
    procedure ExceptionUnwindFunctionEnter(functionId: ULONG); safecall;
    procedure ExceptionUnwindFunctionLeave; safecall;
    procedure ExceptionUnwindFinallyEnter(functionId: ULONG); safecall;
    procedure ExceptionUnwindFinallyLeave; safecall;
    procedure ExceptionCatcherEnter(functionId: ULONG; objectId: ULONG); safecall;
    procedure ExceptionCatcherLeave; safecall;
    // COR_PRF_MONITOR_CLR_EXCEPTIONS
    procedure ExceptionCLRCatcherFound; safecall;
    procedure ExceptionCLRCatcherExecute; safecall;
    // COR_PRF_MONITOR_CCW
    procedure COMClassicVTableCreated(wrappedClassId: ULONG;
      const implementedIID: TGuid; pVTable: Pointer; cSlots: ULONG); safecall;
    procedure COMClassicVTableDestroyed(wrappedClassId: ULONG;
      const implementedIID: TGuid; pVTable: Pointer); safecall;
  end;

implementation

uses
  ComServ, SysUtils, Cor;

{ TBaseProfiler }

procedure TBaseProfiler.AppDomainCreationFinished(appDomainId: ULONG;
  hrStatus: HResult);
begin
end;

procedure TBaseProfiler.AppDomainCreationStarted(appDomainId: ULONG);
begin
end;

procedure TBaseProfiler.AppDomainShutdownFinished(appDomainId: ULONG;
  hrStatus: HResult);
begin
end;

procedure TBaseProfiler.AppDomainShutdownStarted(appDomainId: ULONG);
begin
end;

procedure TBaseProfiler.AssemblyLoadFinished(assemblyId: ULONG;
  hrStatus: HResult);
begin
end;

procedure TBaseProfiler.AssemblyLoadStarted(assemblyId: ULONG);
begin
end;

procedure TBaseProfiler.AssemblyUnloadFinished(assemblyId: ULONG;
  hrStatus: HResult);
begin
end;

procedure TBaseProfiler.AssemblyUnloadStarted(assemblyId: ULONG);
begin
end;

procedure TBaseProfiler.ClassLoadFinished(classId: ULONG;
  hrStatus: HResult);
begin
end;

procedure TBaseProfiler.ClassLoadStarted(classId: ULONG);
begin
end;

procedure TBaseProfiler.ClassUnloadFinished(classId: ULONG;
  hrStatus: HResult);
begin
end;

procedure TBaseProfiler.ClassUnloadStarted(classId: ULONG);
begin
end;

procedure TBaseProfiler.COMClassicVTableCreated(wrappedClassId: ULONG;
  const implementedIID: TGuid; pVTable: Pointer; cSlots: ULONG);
begin
end;

procedure TBaseProfiler.COMClassicVTableDestroyed(wrappedClassId: ULONG;
  const implementedIID: TGuid; pVTable: Pointer);
begin
end;

procedure TBaseProfiler.ExceptionCatcherEnter(functionId,
  objectId: ULONG);
begin
end;

procedure TBaseProfiler.ExceptionCatcherLeave;
begin
end;

procedure TBaseProfiler.ExceptionCLRCatcherExecute;
begin
end;

procedure TBaseProfiler.ExceptionCLRCatcherFound;
begin
end;

procedure TBaseProfiler.ExceptionOSHandlerEnter(__unused: ULONG);
begin
end;

procedure TBaseProfiler.ExceptionOSHandlerLeave(__unused: ULONG);
begin
end;

procedure TBaseProfiler.ExceptionSearchCatcherFound(functionId: ULONG);
begin
end;

procedure TBaseProfiler.ExceptionSearchFilterEnter(functionId: ULONG);
begin
end;

procedure TBaseProfiler.ExceptionSearchFilterLeave;
begin
end;

procedure TBaseProfiler.ExceptionSearchFunctionEnter(functionId: ULONG);
begin
end;

procedure TBaseProfiler.ExceptionSearchFunctionLeave;
begin
end;

procedure TBaseProfiler.ExceptionThrown(thrownObjectId: ULONG);
begin
end;

procedure TBaseProfiler.ExceptionUnwindFinallyEnter(functionId: ULONG);
begin
end;

procedure TBaseProfiler.ExceptionUnwindFinallyLeave;
begin
end;

procedure TBaseProfiler.ExceptionUnwindFunctionEnter(functionId: ULONG);
begin
end;

procedure TBaseProfiler.ExceptionUnwindFunctionLeave;
begin
end;

procedure TBaseProfiler.FunctionUnloadStarted(functionId: ULONG);
begin
end;

function TBaseProfiler.GetAppDomainFromAppDomainId(AppDomainId: ULONG;
  var AppDomainName: WideString): Boolean;
var
  CopiedChars, ProcessID: Cardinal;
  CAppDomainName: array[0..256] of WideChar;
begin
  Result := False;
  try
    CorProfilerInfo.GetAppDomainInfo(AppDomainId, SizeOf(CAppDomainName),
      CopiedChars, CAppDomainName, ProcessID);
    if CopiedChars < SizeOf(CAppDomainName) then
    begin
      AppDomainName := CAppDomainName;
      Result := True
    end
  except
{$ifdef LOG_COR_ERRORS}
    on E: EOleSysError do
      Log('*** GetAppDomainFromAppDomainId error $%x'#10, [E.ErrorCode])
{$endif}
  end
end;

function TBaseProfiler.GetAssemblyFromAssemblyId(AssemblyId: ULONG;
  var AssemblyName: WideString): Boolean;
var
  CopiedChars, AppDomainID, ModuleID: Cardinal;
  CAssemblyName: array[0..256] of WideChar;
begin
  Result := False;
  try
    CorProfilerInfo.GetAssemblyInfo(AssemblyId, SizeOf(CAssemblyName),
      CopiedChars, CAssemblyName, AppDomainID, ModuleID);
    if CopiedChars < SizeOf(CAssemblyName) then
    begin
      AssemblyName := CAssemblyName;
      Result := True
    end
  except
{$ifdef LOG_COR_ERRORS}
    on E: EOleSysError do
      Log('*** GetAssemblyFromAssemblyId error $%x'#10, [E.ErrorCode])
{$endif}
  end
end;

function TBaseProfiler.GetClassAndMethodFromFunctionId(FunctionId: ULONG;
  var ClassName, MethodName: WideString): Boolean;
var
  MethodMetaToken, ClassMetaToken, CopiedChars: Cardinal;
  MetaDataImport: IMetaDataImport;
  CMethodName, CClassName: array[0..256] of WideChar;
begin
  Result := False;
  try
    CorProfilerInfo.GetTokenAndMetaDataFromFunction(
      FunctionId, IMetaDataImport, IUnknown(MetaDataImport), MethodMetaToken);
    MetaDataImport.GetMethodProps(MethodMetaToken, @ClassMetaToken,
      CMethodName, SizeOf(CMethodName), @CopiedChars,
      nil, nil, nil, nil, nil);
    if CopiedChars < SizeOf(CMethodName) then
    begin
      MethodName := CMethodName;
      //Matt Pietrek uses this - not quite sure why...:
      //if ClassMetaToken = $2000000 then
      //  ClassMetaToken := $2000001;
      MetaDataImport.GetTypeDefProps(ClassMetaToken,
        CClassName, SizeOf(CClassName), @CopiedChars, nil, nil);
      if CopiedChars < SizeOf(CClassname) then
      begin
        ClassName := CClassName;
        Result := True
      end
    end
  except
{$ifdef LOG_COR_ERRORS}
    on E: EOleSysError do
      Log('*** GetClassAndMethodFromFunctionId error $%x'#10, [E.ErrorCode])
{$endif}
  end
end;

function TBaseProfiler.GetClassFromClassId(ClassId: ULONG;
  var ClassName: WideString): Boolean;
var
  ClassMetaToken, ModID: Cardinal;
  MetaDataImport: IMetaDataImport;
{$ifdef UNICODE_CLASS_NAME}
  CopiedChars: Cardinal;
  CClassName: array[0..256] of WideChar;
{$else}
  CClassName: PChar;
{$endif}
begin
  Result := False;
  try
    CorProfilerInfo.GetClassIDInfo(ClassID, ModID, ClassMetaToken);
    MetaDataImport := IMetaDataImport(CorProfilerInfo.GetModuleMetaData(
      ModId, ofRead, IMetaDataImport));
{$ifdef UNICODE_CLASS_NAME}
    MetaDataImport.GetTypeDefProps(ClassMetaToken,
      CClassName, SizeOf(CClassName), @CopiedChars, nil, nil);
    if CopiedChars < SizeOf(CClassName) then
    begin
      ClassName := CClassName;
      Result := True
    end
{$else}
    MetaDataImport.GetNameFromToken(ClassMetaToken, CClassName);
    ClassName := CClassName;
    Result := True
{$endif}
  except
{$ifdef LOG_COR_ERRORS}
    on E: EOleSysError do
      Log('*** GetClassFromClassId error $%x'#10, [E.ErrorCode])
{$endif}
  end
end;

function TBaseProfiler.GetClassFromObjectId(ObjectId: ULONG;
  var ClassName: WideString): Boolean;
var
  ClassID: Cardinal;
begin
  Result := False;
  try
    ClassID := CorProfilerInfo.GetClassFromObject(ObjectID);
    Result := GetClassFromClassId(ClassID, ClassName);
  except
{$ifdef LOG_COR_ERRORS}
    on E: EOleSysError do
      Log('*** GetClassFromObjectId error $%x'#10, [E.ErrorCode])
{$endif}
  end
end;

function TBaseProfiler.GetModuleFromModuleId(ModuleId: ULONG;
  var ModuleName: WideString): Boolean;
var
  CopiedChars: Cardinal;
  CModuleName: array[0..MAX_PATH] of WideChar;
begin
  Result := False;
  try
    CorProfilerInfo.GetModuleInfo(ModuleId, nil, SizeOf(CModuleName),
      @CopiedChars, CModuleName, nil);
    if CopiedChars < SizeOf(CModuleName) then
    begin
      ModuleName := CModuleName;
      Result := True
    end
  except
{$ifdef LOG_COR_ERRORS}
    on E: EOleSysError do
      Log('*** GetModuleFromModuleId error $%x'#10, [E.ErrorCode])
{$endif}
  end
end;

procedure TBaseProfiler.Initialize(const pICorProfilerInfoUnk: IUnknown);
begin
{$ifdef NOISE_ON_START_AND_STOP}
  MessageBeep(Cardinal(-1));
  Sleep(100);
  MessageBeep(Cardinal(-1));
{$endif}
  CorProfilerInfo := pICorProfilerInfoUnk as ICorProfilerInfo;
{$ifdef USE_CRITICAL_SECTION}
  InitializeCriticalSectionAndSpinCount(CritSec, 5000);
{$endif}
  EventMask := COR_PRF_MONITOR_NONE;
  OnInitialize;
  CorProfilerInfo.SetEventMask(EventMask);
end;

procedure TBaseProfiler.JITCachedFunctionSearchFinished(
  functionId: ULONG; AResult: COR_PRF_JIT_CACHE);
begin
end;

function TBaseProfiler.JITCachedFunctionSearchStarted(
  functionId: ULONG): Bool;
begin
  Result := True
end;

procedure TBaseProfiler.JITCompilationFinished(functionId: ULONG;
  hrStatus: HResult; fIsSafeToBlock: Integer);
begin
end;

procedure TBaseProfiler.JITCompilationStarted(functionId: ULONG;
  fIsSafeToBlock: Integer);
begin
end;

procedure TBaseProfiler.JITFunctionPitched(functionId: ULONG);
begin
end;

function TBaseProfiler.JITInlining(callerId, calleeId: ULONG): Bool;
begin
  Result := True
end;

procedure TBaseProfiler.Log(const Msg: String);
var
  LogMsg: String;
begin
  LogMsg := StringOfChar(#32, NestingLevel * 2) + Msg + #10;
  OutputDebugString(PChar(LogMsg))
end;

procedure TBaseProfiler.Log(const Msg: String; Args: array of const);
begin
  Log(Format(Msg, Args))
end;

procedure TBaseProfiler.ManagedToUnmanagedTransition(functionId: ULONG;
  reason: COR_PRF_TRANSITION_REASON);
begin
end;

procedure TBaseProfiler.ModuleAttachedToAssembly(moduleId,
  assemblyId: ULONG);
begin
end;

procedure TBaseProfiler.ModuleLoadFinished(moduleId: ULONG;
  hrStatus: HResult);
begin
end;

procedure TBaseProfiler.ModuleLoadStarted(moduleId: ULONG);
begin
end;

procedure TBaseProfiler.ModuleUnloadFinished(moduleId: ULONG;
  hrStatus: HResult);
begin
end;

procedure TBaseProfiler.ModuleUnloadStarted(moduleId: ULONG);
begin
end;

procedure TBaseProfiler.MovedReferences(cMovedObjectIDRanges: ULONG;
  var oldObjectIDRangeStart, newObjectIDRangeStart,
  cObjectIDRangeLength: TULONGArray);
begin
end;

procedure TBaseProfiler.ObjectAllocated(objectId, classId: ULONG);
begin
end;

procedure TBaseProfiler.ObjectReferences(objectId: ULONG; classId: ULONG;
  cObjectRefs: ULONG; var objectRefIds: TULONGArray);
begin
end;

procedure TBaseProfiler.ObjectsAllocatedByClass(cClassCount: ULONG;
  var classIds, cObjects: TULONGarray);
begin
end;

procedure TBaseProfiler.OnInitialize;
begin
end;

procedure TBaseProfiler.OnShutDown;
begin
end;

procedure TBaseProfiler.RemotingClientInvocationFinished;
begin
end;

procedure TBaseProfiler.RemotingClientInvocationStarted;
begin
end;

procedure TBaseProfiler.RemotingClientReceivingReply(const pCookie: TGuid;
  fIsAsync: Bool);
begin
end;

procedure TBaseProfiler.RemotingClientSendingMessage(const pCookie: TGuid;
  fIsAsync: Bool);
begin
end;

procedure TBaseProfiler.RemotingServerInvocationReturned;
begin
end;

procedure TBaseProfiler.RemotingServerInvocationStarted;
begin
end;

procedure TBaseProfiler.RemotingServerReceivingMessage(const pCookie: TGuid;
  fIsAsync: Bool);
begin
end;

procedure TBaseProfiler.RemotingServerSendingReply(const pCookie: TGuid;
  fIsAsync: Bool);
begin
end;

procedure TBaseProfiler.RootReferences(cRootRefs: ULONG;
  var rootRefIds: TULONGArray);
begin
end;

procedure TBaseProfiler.RuntimeResumeFinished;
begin
end;

procedure TBaseProfiler.RuntimeResumeStarted;
begin
end;

procedure TBaseProfiler.RuntimeSuspendAborted;
begin
end;

procedure TBaseProfiler.RuntimeSuspendFinished;
begin
end;

procedure TBaseProfiler.RuntimeSuspendStarted(
  suspendReason: COR_PRF_SUSPEND_REASON);
begin
end;

procedure TBaseProfiler.RuntimeThreadResumed(threadId: ULONG);
begin
end;

procedure TBaseProfiler.RuntimeThreadSuspended(threadId: ULONG);
begin
end;

procedure TBaseProfiler.Shutdown;
begin
  OnShutDown;
{$ifdef USE_CRITICAL_SECTION}
  DeleteCriticalSection(CritSec);
{$endif}
  CorProfilerInfo := nil;
{$ifdef NOISE_ON_START_AND_STOP}
  MessageBeep(Cardinal(-1));
  Sleep(100);
  MessageBeep(Cardinal(-1));
{$endif}
end;

procedure TBaseProfiler.SyncEnter;
begin
{$ifdef USE_CRITICAL_SECTION}
  EnterCriticalSection(CritSec);
{$endif}
end;

procedure TBaseProfiler.SyncExit;
begin
{$ifdef USE_CRITICAL_SECTION}
  LeaveCriticalSection(CritSec);
{$endif}
end;

procedure TBaseProfiler.ThreadAssignedToOSThread(managedThreadId: ULONG;
  osThreadId: ULONG);
begin
end;

procedure TBaseProfiler.ThreadCreated(threadId: ULONG);
begin
end;

procedure TBaseProfiler.ThreadDestroyed(threadId: ULONG);
begin
end;

procedure TBaseProfiler.UnmanagedToManagedTransition(functionId: ULONG;
  reason: COR_PRF_TRANSITION_REASON);
begin
end;

end.
